#!perl -w
use strict;
use PureMessage::Version qw($PREFIX);
die "PureMessage is not properly installed"
    unless $PREFIX && $PREFIX !~ /^%%/;

use lib "./support";
use ActiveState::Install qw(install CFG_FORCE_OURS FILE_ABANDON);
use Config;

our $LIB_DIR = $Config{installsitelib};
our $ARCH_DIR = $Config{installsitearch};

install(
    files => {
        etc => "$PREFIX/etc",
    },
    pkg => "PureMessage-Sophos-Engine",
    ver => "6.0.2",
    etc => "$PREFIX/etc",
    config_files => {
        'etc/data/sophos/filetypes.conf' => CFG_FORCE_OURS,
    },
    special_files => {
        "$ARCH_DIR/auto/Sophos/SAVI/Engine/Engine.so" => FILE_ABANDON,
        "$ARCH_DIR/ActiveState/I18N/UTF8.pm" => FILE_ABANDON,
        "$LIB_DIR/PureMessage/Policy/MessageText.pm" => FILE_ABANDON,
    },
);

{
require ActiveState::Dir::Atomic;

my $sd = "$PREFIX/etc/data/sophos";
my $at = ActiveState::Dir::Atomic->new(
    $sd, writable => 1, create => 1, rotate => 4,
);
my $dd = $at->scratchpath;

# Remove previous scratchpath contents
opendir my $DIR, $dd or die $!;
while(my $file = readdir($DIR)) {
    next if $file eq '.' or $file eq '..';
    next if -d "$dd/$file";
    unlink "$dd/$file" or die $!;
}
closedir $DIR or die $!;

my $libsavi = read_version_file("$sd/libsavi-version");
my $vdl     = read_version_file("$sd/vdl-version");

# create symlinks in $PREFIX/etc/data/sophos
my $so = $Config{so};
force_symlink($vdl, "$sd/vdl.dat") 
    or die "can't symlink $sd/vdl.dat -> $vdl: $!";
force_symlink($libsavi, "$sd/libsavi.$so")
    or die "can't symlink $sd/libsavi.$so -> $libsavi: $!";
force_symlink($libsavi, "$sd/libsavi.$so.3")
    or die "can't symlink $sd/libsavi.$so.3 -> $libsavi: $!";

# hard link files and symlink symlinks from $PREFIX/etc/data/sophos
# into scratchpath
opendir $DIR, $sd or die $!;
while(my $file = readdir($DIR)) {
    next if $file =~ /^\./;
    my $old = "$sd/$file";
    next if -d $old;
    my $new = "$dd/$file";
    unlink($new);
    (-l $old)
        ? (symlink(readlink($old), $new) or die $!)
        : (link($old, $new) or die $!);
}


closedir $DIR or die $!;

$at->commit;
}

my $CURRENT = "$PREFIX/etc/data/sophos/current";
my $DYNALIB = "$PREFIX/lib";
my $so = $Config{so};
for my $file ("libsavi.$so", "libsavi.$so.3") {
    if (-l "$DYNALIB/$file") {
        next if readlink("$DYNALIB/$file") eq "$CURRENT/$file";
        unlink "$DYNALIB/$file";
    }
    elsif (-f "$DYNALIB/$file") {
        unlink "$DYNALIB/$file" or die "can't unlink $DYNALIB/$file: $!";
    }
    symlink("$CURRENT/$file", "$DYNALIB/$file")
        or die "can't symlink $DYNALIB/$file -> $CURRENT/$file: $!";
}

# set permissions on PureMessage
eval {
    require PPM::PureMessage::Utils;
};
unless ($@) {
    my $filetypes = "$PREFIX/etc/data/sophos/filetypes.conf";
    my ($login,$pass,$uid,$gid) = PPM::PureMessage::Utils::pmx_user();
    if (defined $uid && defined $gid) {
	chown $uid, $gid, $filetypes or
	    die "Unable to set file ownership for $filetypes: $!";
    }
    chmod 0444, $filetypes or
	die "Unable to set file permissions for $filetypes: $!";
}

sub force_symlink {
    my ($oldfile, $newfile) = @_;
    if (-l $newfile or -e $newfile) {
        unlink($newfile) or die "can't unlink $newfile: $!";
    }
    symlink($oldfile, $newfile);
}

sub read_version_file {
    my $filename = shift;
    open(my $fh, "<", $filename) or die "Error opening file '$filename': $!";
    my $version = <$fh>;
    close $fh;
    return $version;
}

1;
